package org.simantics.objmap.examples;

import java.util.Collection;

import org.simantics.db.ReadGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.IMappingSchema;
import org.simantics.objmap.annotations.GraphType;
import org.simantics.objmap.annotations.RelatedElement;
import org.simantics.objmap.annotations.RelatedElements;
import org.simantics.objmap.annotations.RelatedValue;
import org.simantics.objmap.schema.MappingSchemas;
import org.simantics.objmap.schema.SimpleSchema;

public class SysdynExample {

    @GraphType("http://www.simantics.org/Sysdyn#Configuration")
    static class Configuration {
        @RelatedElements("http://www.vtt.fi/Simantics/Layer0/1.0/Relations#ConsistsOf")
        Collection<Component> components; 
    }
    
    static abstract class Component {        
    }
    
    @GraphType("http://www.simantics.org/Sysdyn#Dependency")
    static class Dependency extends Component {
        @RelatedElement("http://www.simantics.org/Sysdyn#HasTail")
        Variable tail;
        @RelatedElement("http://www.simantics.org/Sysdyn#HasHead")
        Auxiliary head;
    }
    
    static abstract class Variable extends Component {
        @RelatedValue("http://www.vtt.fi/Simantics/Layer0/1.0/Relations#HasName")
        String name;
    }
    
    @GraphType("http://www.simantics.org/Sysdyn#Auxiliary")
    static class Auxiliary extends Variable {
    }
    
    public static IMappingSchema createSchema(ReadGraph g) throws DatabaseException, InstantiationException, IllegalAccessException {
        SimpleSchema schema = new SimpleSchema();
        schema.addLinkType(MappingSchemas.fromAnnotations(g, Configuration.class));
        schema.addLinkType(MappingSchemas.fromAnnotations(g, Dependency.class));
        schema.addLinkType(MappingSchemas.fromAnnotations(g, Auxiliary.class));
        return schema;
    }   
    
}
