package org.simantics.objmap;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;

/**
 * A rule for how domain and range elements are related to each other. 
 * @author Hannu Niemist
 */
public interface IMappingRule {
    /**
     * Modifies the domain element so that it corresponds to the range element.
     * @param g write transaction
     * @param map unidirectional view of the current mapping
     * @param domainElement the domain element that is updated
     * @param rangeElement the range element that corresponds to the domain element
     * @return true if the rule made some modifications
     * @throws MappingException 
     */    
	boolean updateDomain(WriteGraph g, IFunction<Object, Resource> map, Resource domainElement, Object rangeElement) throws MappingException;

	/**
	 * Modifies the range element so that it corresponds to the domain element.
	 * @param g read transaction
	 * @param map unidirectional view of the current mapping
	 * @param domainElement the domain element that corresponds to the range element
	 * @param rangeElement the range element that is updated
	 * @return true if the rule made some modifications
	 * @throws MappingException
	 */
	boolean updateRange(ReadGraph g, IFunction<Resource, Object> map, Resource domainElement, Object rangeElement) throws MappingException;
}
