package org.simantics.objmap;

import org.simantics.objmap.impl.Mapping;
import org.simantics.objmap.impl.UnidirectionalMapping;

/**
 * Static utility methods for mappings. 
 * @author Hannu Niemist
 */
public class Mappings {
	private Mappings() {}
	
	/**
	 * Creates a new mapping based on the given mapping schema. 
	 * The created mapping is not thread-safe and will not
	 * listen database changes automatically.
	 */
	public static IMapping createWithoutListening(IMappingSchema schema) {
		return new Mapping(schema, false);
	}
	
	/**
     * Creates a new mapping based on the given mapping schema. 
     * The created mapping is not thread-safe. It listens database
     * changes automatically.
     */
	public static IMapping createWithListening(IMappingSchema schema) {
        return new Mapping(schema, true);
    }
	
	/**
	 * Creates a mapping that supports only the direction from domain to range.
	 * Does not listen the database.
	 */
	public static IMapping createUnidirectional(IMappingSchema schema) {
	    return new UnidirectionalMapping(schema);
	}
}
