package org.simantics.objmap.impl;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.SyncListener;
import org.simantics.db.request.Read;
import org.simantics.objmap.IFunction;
import org.simantics.objmap.MappingException;

public class RangeUpdateRequest implements Read<Boolean>, SyncListener<Boolean> {

    Link link;
    /*
     * Note that this map uses a read request that it has got from caller and 
     * not the one that is used in updateRange. This is intentional.
     */
    IFunction<Resource, Object> map; // map==null is used to flag that request is performed once
    Mapping mapping; // mapping==null is used as a flag the request disposed
    
    public RangeUpdateRequest(Link link, IFunction<Resource, Object> map, Mapping mapping) {
        this.link = link;
        this.map = map;
        this.mapping = mapping;
    }

    @Override
    public Boolean perform(ReadGraph g) throws DatabaseException {
        if(map != null) {
            link.type.updateRange(g, map, link.domainElement, link.rangeElement);
            map = null;
            return Boolean.TRUE;
        }
        else if(mapping != null) {
            mapping.domainModified(link);
            mapping = null;
            return Boolean.FALSE;
        }
        else
            return null;
    }
    
    @Override
    public void exception(ReadGraph graph, Throwable throwable)
            throws DatabaseException {
        if(throwable instanceof DatabaseException)
            throw (DatabaseException)throwable;
        else
            throw new MappingException(throwable);
    }

    @Override
    public void execute(ReadGraph graph, Boolean result)
            throws DatabaseException {       
    }

    @Override
    public boolean isDisposed() {
        return mapping == null || link.removed || mapping.isDisposed();
    }
    
    
    
}
