package org.simantics.objmap.rules;

import java.util.ArrayList;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.objmap.IFunction;
import org.simantics.objmap.IMappingRule;
import org.simantics.objmap.MappingException;
import org.simantics.objmap.rules.domain.IDomainAccessor;
import org.simantics.objmap.rules.range.IRangeAccessor;

/**
 * A rule that synchronizes collection of elements between
 * domain and range accessors. Elements are mapped from
 * between domain and range during the synchronization.
 * @author Hannu Niemist
 */
public class MappedElementRule implements IMappingRule {
	IDomainAccessor<Resource> domainAccessor;
	IRangeAccessor<Object> rangeAccessor;
	
	public MappedElementRule(IDomainAccessor<Resource> domainAccessor,
            IRangeAccessor<Object> rangeAccessor) {
        this.domainAccessor = domainAccessor;
        this.rangeAccessor = rangeAccessor;
    }

    @Override
	public boolean updateDomain(WriteGraph g, IFunction<Object, Resource> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
	    Object value = rangeAccessor.get(rangeElement);
	    Resource mappedValue = value == null ? null : map.get(value);
		return domainAccessor.set(g, domainElement, mappedValue);
	}

	@Override
	public boolean updateRange(ReadGraph g, IFunction<Resource, Object> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
	    Resource value = domainAccessor.get(g, domainElement);
        Object mappedValue = value == null ? null : map.get(value);
        return rangeAccessor.set(rangeElement, mappedValue);
	}	
}
