package org.simantics.objmap.rules;

import java.util.ArrayList;
import java.util.Collection;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.objmap.IFunction;
import org.simantics.objmap.IMappingRule;
import org.simantics.objmap.MappingException;
import org.simantics.objmap.rules.domain.IDomainAccessor;
import org.simantics.objmap.rules.range.IRangeAccessor;

/**
 * A rule that synchronizes collection of elements between
 * domain and range accessors. Elements are mapped from
 * between domain and range during the synchronization.
 * @author Hannu Niemist
 */
public class MappedElementsRule implements IMappingRule {
	IDomainAccessor<Collection<Resource>> domainAccessor;
	IRangeAccessor<Collection<Object>> rangeAccessor;
	
	public MappedElementsRule(IDomainAccessor<Collection<Resource>> domainAccessor,
			IRangeAccessor<Collection<Object>> rangeAccessor) {
		this.domainAccessor = domainAccessor;
		this.rangeAccessor = rangeAccessor;
	}

	@Override
	public boolean updateDomain(WriteGraph g, IFunction<Object, Resource> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
	    Collection<Object> value = rangeAccessor.get(rangeElement);
	    ArrayList<Resource> mappedValue = new ArrayList<Resource>(value.size());
	    for(Object obj : value)
	        mappedValue.add(map.get(obj));
		return domainAccessor.set(g, domainElement, mappedValue);
	}

	@Override
	public boolean updateRange(ReadGraph g, IFunction<Resource, Object> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
	    Collection<Resource> value = domainAccessor.get(g, domainElement);
        ArrayList<Object> mappedValue = new ArrayList<Object>(value.size());
        for(Resource r : value)
            mappedValue.add(map.get(r));
        return rangeAccessor.set(rangeElement, mappedValue);
	}	
}
