package org.simantics.objmap.rules;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.objmap.IFunction;
import org.simantics.objmap.IMappingRule;
import org.simantics.objmap.MappingException;
import org.simantics.objmap.rules.domain.IDomainAccessor;
import org.simantics.objmap.rules.range.IRangeAccessor;

/**
 * A rule that synchronizes values between domain and
 * range accessors.
 * @author Hannu Niemist
 */
public class ValueRule implements IMappingRule {
	IDomainAccessor<Object> domainAccessor;
	IRangeAccessor<Object> rangeAccessor;
	
	public ValueRule(IDomainAccessor<Object> domainAccessor,
			IRangeAccessor<Object> rangeAccessor) {
		this.domainAccessor = domainAccessor;
		this.rangeAccessor = rangeAccessor;
	}

	@Override
	public boolean updateDomain(WriteGraph g, IFunction<Object, Resource> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
		Object value = rangeAccessor.get(rangeElement);
		return domainAccessor.set(g, domainElement, value);
	}

	@Override
	public boolean updateRange(ReadGraph g, IFunction<Resource, Object> map,
			Resource domainElement, Object rangeElement)
			throws MappingException {
		Object value = domainAccessor.get(g, domainElement);
		return rangeAccessor.set(rangeElement, value);
	}	
}
