package org.simantics.objmap.rules.domain;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.MappingException;

/**
 * Accesses a resource attached to the element by given functional relation.
 * @author Hannu Niemist
 */
public class RelatedObjectAccessor implements IDomainAccessor<Resource> {

	Resource relation;
	
	public RelatedObjectAccessor(Resource relation) {
		this.relation = relation;
	}

	@Override
	public Resource get(ReadGraph g, Resource element) throws MappingException {
		try {
			return g.getPossibleObject(element, relation);
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
	}
	
	@Override
	public boolean set(WriteGraph g, Resource element, Resource value)
			throws MappingException {
		try {
		    Resource resource = g.getPossibleObject(element, relation);
			if(resource == null) {
			    if(value == null)
			        return false;
			    g.claim(element, relation, value);
			    return true;
			}
			else if(resource.equals(value))
			    return false;
			else {
			    g.deny(element, relation);
			    if(value != null)
			        g.claim(element, relation, value);
			    return true;
			}
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
		
	}

}
