package org.simantics.objmap.rules.domain;

import java.util.Collection;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.MappingException;

/**
 * Accesses the set of objects attached to the element by the given relation.
 * @author Hannu Niemist
 */
public class RelatedObjectsAccessor implements IDomainAccessor<Collection<Resource>> {

	Resource relation;

	public RelatedObjectsAccessor(Resource relation) {
		this.relation = relation;
	}

	@Override
	public Collection<Resource> get(ReadGraph g, Resource element) throws MappingException {
		try {
			return g.getObjects(element, relation);
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
	}
	
	@Override
	public boolean set(WriteGraph g, Resource element, Collection<Resource> value)
			throws MappingException {
		try {
			return MappingUtils.synchronizeStatements(g, element, relation, value.toArray(new Resource[value.size()]));
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
		
	}

}
