package org.simantics.objmap.rules.domain;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.MappingException;

/**
 * Accesses a value attached to the element by given functional relation.
 * @author Hannu Niemist
 */
public class RelatedValueAccessor implements IDomainAccessor<Object> {

	Resource relation;
	Resource valueType;

	public RelatedValueAccessor(Resource relation, Resource valueType) {
		this.relation = relation;
		this.valueType = valueType;
	}

	@Override
	public Object get(ReadGraph g, Resource element) throws MappingException {
		try {
			Resource valueResource = g.getPossibleObject(element, relation);
			if(valueResource == null)
				return null;
			return g.getValue(valueResource);
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
	}
	
	@Override
	public boolean set(WriteGraph g, Resource element, Object value)
			throws MappingException {
		try {
			Resource valueResource = g.getPossibleObject(element, relation);
			if(valueResource == null) {
				if(value == null)
					return false;
				valueResource = g.newResource();
				g.claim(valueResource, g.getBuiltins().InstanceOf, 
						valueType);
				g.claim(element, relation, valueResource);
				g.claimValue(valueResource, value);				
				return true;
			}
			else {
				if(value == null) {
					g.deny(valueResource);
					return true;
				}				
				Object currentValue = g.getValue(valueResource);
				if(currentValue.equals(value))
					return false;
				g.claimValue(valueResource, value);
				return true;
			}
		} catch (DatabaseException e) {
			throw new MappingException(e);
		}
		
	}

}
