package org.simantics.objmap.rules.range;

import java.lang.reflect.Field;

import org.simantics.objmap.MappingException;

/**
 * Accesses the given field of the element.
 * @author Hannu Niemist
 */
public class FieldAccessor<T> implements IRangeAccessor<T> {
	Field field;

	public FieldAccessor(Field field) {
		this.field = field;
	}

	@Override
	public T get(Object element) throws MappingException {
		try {
			return (T)field.get(element);
		} catch (IllegalArgumentException e) {
			throw new MappingException(e);
		} catch (IllegalAccessException e) {
			throw new MappingException(e);
		}
	}

	@Override
	public boolean set(Object element, T value) throws MappingException {
		try {
			Object currentValue = field.get(element);
			if(currentValue == null ? value == null : currentValue.equals(value))
				return false;
			field.set(element, value);
			return true;
		} catch (IllegalArgumentException e) {
			throw new MappingException(e);
		} catch (IllegalAccessException e) {
			throw new MappingException(e);
		}
	}	
}
