package org.simantics.objmap.schema;

import gnu.trove.THashMap;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.exception.DatabaseException;
import org.simantics.layer0.utils.direct.GraphUtils;
import org.simantics.objmap.ILinkType;
import org.simantics.objmap.IMappingSchema;
import org.simantics.objmap.MappingException;

/**
 * An implementation of IMappingSchema that contains
 * only SimpleLinkTypes. The link type of any domain
 * element is based solely on its type in database and
 * the link type of any range element is based on its class.
 * @author Hannu Niemist
 */
public class SimpleSchema implements IMappingSchema {

    THashMap<Resource, SimpleLinkType> domainLinkTypes = 
        new THashMap<Resource, SimpleLinkType>();
    THashMap<Class<?>, SimpleLinkType> rangeLinkTypes = 
        new THashMap<Class<?>, SimpleLinkType>();
    
    public void addLinkType(SimpleLinkType linkType) {
        domainLinkTypes.put(linkType.domainType, linkType);
        rangeLinkTypes.put(linkType.rangeType, linkType);
    }
    
    @Override
    public ILinkType linkTypeOfDomainElement(ReadGraph g, Resource element) throws MappingException {        
        try {
            ILinkType type = domainLinkTypes.get(
                    g.getSingleObject(element, g.getBuiltins().InstanceOf));
            if(type == null) 
   				throw new MappingException("Didn't find a link type for " +
   						GraphUtils.getReadableName(g, element) + "."
   						);
    		return type;
        } catch (DatabaseException e) {
            throw new MappingException(e);
        }
    }

    @Override
    public ILinkType linkTypeOfRangeElement(Object element) throws MappingException {
    	ILinkType type = rangeLinkTypes.get(element.getClass());
		if(type == null) 
			throw new MappingException("Didn't find a link type for " +	element + ".");
		return type;
    }

}
